@@ -2,16 +2,12 @@
 ** fscheck
 ** The Sleuth Kit 
 **
-** $Date: 2007/12/18 22:43:31 $
-**
-** Brian Carrier [carrier@sleuthkit.org]
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved 
 **
-**
 ** This software is distributed under the Common Public License 1.0
-**
 */
-#include "tsk/tsk_tools_i.h"
+#include "tsk3/tsk_tools_i.h"
 
 static void
 usage()
@@ -35,31 +31,68 @@ usage()
 int
 main(int argc, char **argv)
 {
-    char *fstype = NULL;
+    TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
+    TSK_IMG_INFO *img;
+
+    TSK_OFF_T imgoff = 0;
+    TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
+    TSK_FS_INFO *fs;
+
     int ch;
-    FS_INFO *fs;
-    char *imgtype = NULL, *imgoff = NULL;
-    IMG_INFO *img;
+    TSK_TCHAR **argv;
+    
+#ifdef TSK_WIN32
+    // On Windows, get the wide arguments (mingw doesn't support wmain)
+    argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+    if( argv == NULL) {    
+        fprintf(stderr, "Error getting wide arguments\n");
+        exit(1);
+    }
+#else
+    argv = (TSK_TCHAR **)argv1;
+#endif
+    
 
     progname = argv[0];
 
-    while ((ch = getopt(argc, argv, "f:i:o:vV")) > 0) {
+    while ((ch = GETOPT(argc, argv, "f:i:o:vV")) > 0) {
         switch (ch) {
         case '?':
         default:
-            fprintf(stderr, "Invalid argument: %s\n", argv[optind]);
+            fprintf(stderr, "Invalid argument: %s\n", argv[OPTIND]);
             usage();
 
         case 'f':
-            fstype = optarg;
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_fs_type_print(stderr);
+                exit(1);
+            }
+            fstype = tsk_fs_type_toid(OPTARG);
+            if (fstype == TSK_FS_TYPE_UNSUPP) {
+                TFPRINTF(stderr,
+                    _TSK_T("Unsupported file system type: %s\n"), OPTARG);
+                usage();
+            }
             break;
 
         case 'i':
-            imgtype = optarg;
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_img_type_print(stderr);
+                exit(1);
+            }
+            imgtype = tsk_img_type_toid(OPTARG);
+            if (imgtype == TSK_IMG_TYPE_UNSUPP) {
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
 
         case 'o':
-            imgoff = optarg;
+            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
+                tsk_error_print(stderr);
+                exit(1);
+            }
             break;
 
         case 'v':
@@ -73,14 +106,14 @@ main(int argc, char **argv)
     }
 
     /* We need at least one more argument */
-    if (optind >= argc) {
+    if (OPTIND >= argc) {
         fprintf(stderr, "Missing image name\n");
         usage();
     }
 
     img =
-        img_open(imgtype, imgoff, argc - optind,
-        (const char **) &argv[optind]);
+        img_open(imgoff, argc - OPTIND,
+        (const char **) &argv[OPTIND], imgtype);
     if (img == NULL) {
         tsk_error_print(stderr);
         exit(1);
