@@ -2,10 +2,8 @@
 ** icat 
 ** The Sleuth Kit 
 **
-** $Date: 2007/12/20 16:17:47 $
-**
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
@@ -22,9 +20,9 @@
  *	P.O. Box 704
  *	Yorktown Heights, NY 10598, USA
  --*/
-#include "tsk/tsk_tools_i.h"
-#include <locale.h>
 
+#include "tsk3/tsk_tools_i.h"
+#include <locale.h>
 
 /* usage - explain and terminate */
 
@@ -55,90 +53,113 @@ usage()
 }
 
 int
-MAIN(int argc, TSK_TCHAR ** argv)
+main(int argc, char ** argv1)
 {
-    TSK_TCHAR *fstype = NULL;
-    TSK_TCHAR *imgtype = NULL;
-    TSK_FS_INFO *fs;
+    TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_IMG_INFO *img;
-    TSK_INUM_T inum;
-    int flags = 0;
-    int ch;
-    uint32_t type = 0;
-    uint16_t id = 0;
-    int id_used = 0;
-    int retval;
+
     TSK_OFF_T imgoff = 0;
+    TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
+    TSK_FS_INFO *fs;
+
+    TSK_INUM_T inum;
+    int fw_flags = 0;
+    int ch;
+    TSK_FS_ATTR_TYPE_ENUM type = TSK_FS_ATTR_TYPE_DEFAULT;
+    uint16_t id = 0;
+    uint8_t id_used = 0, type_used = 0;
+    int retval;
     int suppress_recover_error = 0;
+    TSK_TCHAR **argv;
+    
+#ifdef TSK_WIN32
+    // On Windows, get the wide arguments (mingw doesn't support wmain)
+    argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+    if( argv == NULL) {    
+        fprintf(stderr, "Error getting wide arguments\n");
+        exit(1);
+    }
+#else
+    argv = (TSK_TCHAR **)argv1;
+#endif    
 
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _TSK_T("f:hi:o:rRsvV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("f:hi:o:rRsvV"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
             TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
-                argv[optind]);
+                argv[OPTIND]);
             usage();
         case _TSK_T('f'):
-            fstype = optarg;
-            if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
-                tsk_fs_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_fs_type_print(stderr);
                 exit(1);
             }
+            fstype = tsk_fs_type_toid(OPTARG);
+            if (fstype == TSK_FS_TYPE_UNSUPP) {
+                TFPRINTF(stderr,
+                    _TSK_T("Unsupported file system type: %s\n"), OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('h'):
-            flags |= TSK_FS_FILE_FLAG_NOSPARSE;
+            fw_flags |= TSK_FS_FILE_WALK_FLAG_NOSPARSE;
             break;
         case _TSK_T('i'):
-            imgtype = optarg;
-            if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
-                tsk_img_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_img_type_print(stderr);
                 exit(1);
             }
+            imgtype = tsk_img_type_toid(OPTARG);
+            if (imgtype == TSK_IMG_TYPE_UNSUPP) {
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(optarg)) == -1) {
+            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
             break;
         case _TSK_T('r'):
-            flags |= TSK_FS_FILE_FLAG_RECOVER;
+            // this is no longer needed, so we silently ignore it. 
             break;
         case _TSK_T('R'):
-            flags |= TSK_FS_FILE_FLAG_RECOVER;
             suppress_recover_error = 1;
             break;
         case _TSK_T('s'):
-            flags |= TSK_FS_FILE_FLAG_SLACK;
+            fw_flags |= TSK_FS_FILE_WALK_FLAG_SLACK;
             break;
         case _TSK_T('v'):
             tsk_verbose++;
             break;
         case _TSK_T('V'):
-            tsk_print_version(stdout);
+            tsk_version_print(stdout);
             exit(0);
         }
     }
 
     /* We need at least two more argument */
-    if (optind + 1 >= argc) {
+    if (OPTIND + 1 >= argc) {
         tsk_fprintf(stderr, "Missing image name and/or address\n");
         usage();
     }
 
     /* Get the inode address */
-    if (tsk_parse_inum(argv[argc - 1], &inum, &type, &id, &id_used)) {
+    if (tsk_fs_parse_inum(argv[argc - 1], &inum, &type, &type_used, &id, &id_used)) {
         TFPRINTF(stderr, _TSK_T("Invalid inode address: %s\n"),
             argv[argc - 1]);
         usage();
     }
 
     if ((img =
-            tsk_img_open(imgtype, argc - optind - 1,
-                (const TSK_TCHAR **) &argv[optind])) == NULL) {
+            tsk_img_open(argc - OPTIND - 1,
+                (const TSK_TCHAR **) &argv[OPTIND], imgtype)) == NULL) {
         tsk_error_print(stderr);
         exit(1);
     }
@@ -148,10 +169,10 @@ MAIN(int argc, TSK_TCHAR ** argv)
             PRIu64 ")\n", img->size / 512);
         exit(1);
     }
-    if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
+    if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
         tsk_error_print(stderr);
         if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
-            tsk_fs_print_types(stderr);
+            tsk_fs_type_print(stderr);
         img->close(img);
         exit(1);
     }
@@ -173,14 +194,7 @@ MAIN(int argc, TSK_TCHAR ** argv)
         exit(1);
     }
 
-    if (id_used)
-        retval = tsk_fs_icat(fs, 0, inum, type, id, flags);
-    /* If the id value was not used, then set the flag accordingly so the '0' value is ignored */
-    else
-        retval =
-            tsk_fs_icat(fs, 0, inum, type, id,
-            flags | TSK_FS_FILE_FLAG_NOID);
-
+    retval = tsk_fs_icat(fs, inum, type, type_used, id, id_used, (TSK_FS_FILE_WALK_FLAG_ENUM)fw_flags);
     if (retval) {
         if ((suppress_recover_error == 1)
             && (tsk_errno == TSK_ERR_FS_RECOVER)) {
